#ifndef BIBLI_H

#define BIBLI_H

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <assert.h>
#include <math.h>

#define P2 "P2"
#define P1 "P1"

#define COLORS uint8_t

//on passe à la ligne dans les fichiers PGM après MAXCOL colonnes :
#define MAXCOL 30


/*
------------------------------------------------
ALLOCATIONS
------------------------------------------------
*/

//fichier alloc.c 
void * mymalloc(size_t size);
void myfree(void *ptr);
void * mycalloc( size_t elementCount, size_t elementSize );
extern int nballoc;
extern int nbfree;

/*
------------------------------------------------
STRUCTURES
------------------------------------------------
 */
typedef struct image img;

//la couleur est entière dans cette structure !!
struct image{
  int width;//nb colonnes
  int height;//nb lignes.
  COLORS * colors;//tableau de width * height couleurs
  char * magic;//nombre magique
  COLORS max;
};

typedef struct point pt;

//les coords et la couleur sont flottants dans cette structure !!
struct point{
  float color;//la couleur
  float x; //abscisse
  float y;// ordonnée
};

//ligne brisée
typedef struct line line;

struct line{
  int card;// nb de points
  pt * sommets;//sommets de la ligne brisée
};


/*
---------------------------------------------
  FONCTIONS DU DEVOIR
---------------------------------------------
 */

//définies dans divers.c

/*
Attention : 
  - dans une image au format P2, 0 est la couleur noire
  - dans une image au format P1, 0 est la couleur blanche !
*/
img* uni(int w, int h, bool gray, COLORS maxi);
void display_img(img *);//mme rendu qu'un fichier pgm
void free_img(img *);//libérer l'image et les pointeurs
img * fromfile(char *);// img à partir d'un fichier
void tofile(char*, img *);// fichier à partir d'une image
img * neg(img *);// négatif d'une image
img * seuil (img * im, COLORS blanc);//renvoie une image P1 à partir d'une image P2
img * symv(img *im);//inverse l'ordre des colonnes
img * symh(img *im);//inverse l'ordre des lignes


// définies dans polygon.c
void draw_line(img *im, line *l);// trace une ligne brisée 
void display_line_py(char* varname,line * l);//prof. Python like
void display_point(pt p);// prof Python like

void free_line(line * l);

/*
p : un point
absmax : abscisse max de l'image
ordmax : ordonnée max de l'image
HYP : le point p est supposé dans les limites de l'image, 
éventuellement sur les bords. Erreur d'assertion sinon.
RENVOIE : le point de coordonnées entières le plus proche de p
*/
pt proche(pt p, int absmax, int ordmax);


/*
p0, p1 : 2 points
RENVOIE le point image de p2 par la rotation de centre p0 et d'angle +Pi/3
On rappelle l'existence de M_PI dans math.h
*/
pt third(pt p0, pt p1);

/*
l : pointeur sur ligne brisée
Chaque segment A____B de deux points A,B consécutifs dans l

devient une ligne A__/\__B avec 3 points supplémentaires.
Effets de bords : *l est modifiée
*/
void maj(line * l);

//img * histo(); FFF
line * koch(int n, pt p0, pt p1);


/*
------------------------------------------------
FONCTION OUTIL de TRACE de SEGMENT (donnée)
------------------------------------------------
*/

void trace_segment(img *im, pt p0, pt p1);

/*
  ----------------------------------------------
FONCTIONS SPECIALES TESTS PROFS
------------------------------------------------
*/

//sortie sous forme de matrice python
void display_img_py(img * im, char * dest, char* varname, char * mode);

#endif
